#version 300 es
//Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

precision highp float;

in vec3 Position;
in vec3 Normal;
in vec2 TexCoord;

out vec4 FragColor;              
// IBL
uniform samplerCube IBLSpecularMap;
uniform samplerCube IBLDiffuseMap;
uniform sampler2D   IBLBRDFLUT;

//PBR material maps
uniform sampler2D Tex0; // base color
uniform vec4 ColorFactor;

uniform sampler2D Tex1; // metallic roughness
uniform float MetallicFactor;
uniform float RoughnessFactor;

#ifdef HAS_NORMALMAP
uniform sampler2D Tex2; // normal
#endif

#ifdef HAS_OCCLUSIONMAP
uniform sampler2D Tex3; // occlusion
#endif
uniform float OcclusionFactor;

#ifdef HAS_EMISSIONMAP
uniform sampler2D Tex4; // emission
uniform vec3 EmissiveFactor;
#endif

struct MaterialInfo
{
  float d; // Dissolve, 1 means opaque. Also used in mask where glTF model provides the cutoff alpha value, default 0.5
};
uniform MaterialInfo Material;

// Camera Position
uniform vec3 CameraPosition;

const float PI = 3.14159265359;

const float Exposure = 0.9;

vec3 getNormalFromMap()
{
  vec3 pos_dx = dFdx(Position);
  vec3 pos_dy = dFdy(Position);
  vec3 tex_dx = dFdx(vec3(TexCoord, 0.0));
  vec3 tex_dy = dFdy(vec3(TexCoord, 0.0));
  vec3 t      = (tex_dy.t * pos_dx - tex_dx.t * pos_dy) / (tex_dx.s * tex_dy.t - tex_dy.s * tex_dx.t);
  
  vec3 N = normalize(Normal);
  vec3 T = normalize(t - N * dot(N, t));
  vec3 B = normalize(cross(N, T));
  mat3 TBN = mat3(T, B, N);

#ifdef HAS_NORMALMAP
#ifdef RG_NORMALMAP
  vec3 tangentNormal;
  tangentNormal.xy = texture(Tex2, TexCoord).xy * 2.0 - 1.0;
  tangentNormal.z = sqrt(1.0 - clamp(dot(tangentNormal.xy, tangentNormal.xy), 0.0, 1.0));
#else
  vec3 tangentNormal = texture(Tex2, TexCoord).xyz * 2.0 - 1.0;
#endif
  return normalize(TBN * tangentNormal);
#else
  return normalize(TBN[2].xyz);
#endif  
}

vec3 fresnelSchlickRoughness(float cosTheta, vec3 F0, float roughness)
{
  return F0 + (max(vec3(1.0 - roughness), F0) - F0) * pow(clamp(1.0 - cosTheta, 0.0, 1.0), 5.0);
}

float getMipMapLevel(vec2 texUV)
{
  vec2 dx = dFdx(texUV);
  vec2 dy = dFdy(texUV);
  float delta_max_sqr = max(dot(dx, dx), dot(dy, dy));
  return 0.5 + log2(delta_max_sqr);
}

vec3 getSpecularLight(vec3 reflectDir, float level)
{
  vec3 s0 = textureLod(IBLSpecularMap, reflectDir, level).rgb * Exposure;
  return s0;
}

vec3 tonemapReinhard(vec3 color) {
  return color / (color + vec3(1.0));
}

vec3 tonemapWhiteLumaBasedReinhard(vec3 color) {
  float L = dot(color, vec3(0.2126, 0.7152, 0.0722));
  float white = 0.85;
  float nL = L * (1.0 + L / (white * white)) / (1.0 + L);
  float scale = nL / L;
  return color * scale;
}

vec3 tonemapJimRichard(vec3 color) {
  vec3 x = max(vec3(0.0), color - vec3(0.004));
  return x * (6.2 * x + vec3(0.5)) / (x * (6.2 * x + vec3(1.7)) + vec3(0.006));
}

vec3 tonemapUncharted(vec3 x) {
//https://github.com/Zackin5/Filmic-Tonemapping-ReShade/blob/master/Uncharted2.fx
  float A = 0.22;
  float B = 0.30;
  float C = 0.10;
  float D = 0.20;
  float E = 0.02;
  float F = 0.30;
  
  return((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - vec3(E/F);
}

vec3 unchartedColorAdjust(vec3 color){
  float w = 11.2;
  float exposureBias = 2.0;
  vec3 whiteScale = vec3(1.0f)/tonemapUncharted(vec3(w));
  return tonemapUncharted(color * exposureBias) * whiteScale;
}


vec3 toGamma(vec3 color)
{
  return pow(color, vec3(1.0/2.2));
}

void main(void)
{
  const float MAX_REFLECTION_LOD = 9.0; // Depending on the max image size of specular LOD texture: 512 x 512.
  vec3 N = getNormalFromMap();
  vec3 V = normalize(CameraPosition - Position);
  vec3 R = -normalize(reflect(V, N));
  
#ifdef HAS_ALPHAMASK
  vec4 alphaColor = textureLod(Tex0, TexCoord, 0.0) * ColorFactor;
  if(alphaColor.a < Material.d) {
    discard;
  }
#endif
  
  vec4 baseColor = texture(Tex0, TexCoord) * ColorFactor;
  
#ifdef KHR_MATERIALS_UNLIT
  FragColor = baseColor; // If it is unlit, just take the base color, no other light computation.
  return;
#else
  
  vec3 albedo = texture(Tex1, TexCoord).rgb;
  float metallic = clamp(albedo.b * MetallicFactor, 0.0, 1.0);
  float roughness = clamp(albedo.g * RoughnessFactor, 0.04, 1.0);
  albedo = pow(baseColor.rgb, vec3(2.2));
  
  vec3 F0 = vec3(0.04);
  vec3 irradiance = texture(IBLDiffuseMap, N).rgb * Exposure;
  vec3 diffuse = (vec3(1.0) - F0) * albedo * (1.0 - metallic) * irradiance;
  F0 = mix(F0, albedo, metallic);
  
  // ambient lighting using IBL
 /* vec3 F = fresnelSchlickRoughness(max(dot(N, V), 0.0), F0, roughness);
  vec3 kS = F;
  vec3 kD = 1.0 - kS;
  kD *= 1.0 - metallic;*/
  
  // specular part calculation.
  vec3 specularLight = getSpecularLight(R, roughness * MAX_REFLECTION_LOD);
  vec3 brdf = texture(IBLBRDFLUT, vec2(max(abs(dot(N, V)), 0.0), roughness)).rgb;
  vec3 specular = specularLight * (F0 * brdf.x + brdf.y);
  
#ifdef HAS_OCCLUSIONMAP
  vec3 ao = texture(Tex3, TexCoord).rrr; // In case that ao is written in the r channel only.
#elif defined(HAS_MERGEDOCCLUSIONMAP)
  vec3 ao = texture(Tex1, TexCoord).rrr; // It is merged into metallic roughness.
#else
  vec3 ao = vec3(1.0);
#endif
  ao = mix(vec3(1.0), ao, OcclusionFactor);
  vec3 ambient = (diffuse + specular) * ao;
  
  vec3 color = ambient;
  
  // Added emissive
#ifdef HAS_EMISSIONMAP
  vec3 emissive = pow(texture(Tex4, TexCoord).rgb * EmissiveFactor, vec3(2.2));
  color += emissive;
#endif
  
  // Tone mapping
  color = tonemapWhiteLumaBasedReinhard(color);//tonemapJimRichard(color);//tonemapReinhard(color);
  
  // gamma correction
  color = toGamma(color);
  
  FragColor = vec4(color, baseColor.a);
#endif // unlit case.
}